package GUTI;

import javax.annotation.processing.SupportedOptions;
import javax.lang.model.element.AnnotationMirror;

import GUT.GUTAnnotatedTypeFactory;
import GUT.GUTChecker;
import GUT.quals.Any;
import GUT.quals.Bottom;
import GUT.quals.Lost;
import GUT.quals.Peer;
import GUT.quals.Rep;
import GUT.quals.Self;
import checkers.inference.InferenceTypeChecker;
import checkers.quals.TypeQualifiers;
import checkers.source.SupportedLintOptions;
import checkers.types.AnnotatedTypeFactory;
import checkers.types.AnnotatedTypeMirror;
import checkers.types.AnnotatedTypeMirror.AnnotatedNullType;
import checkers.types.AnnotatedTypeMirror.AnnotatedPrimitiveType;
import checkers.types.AnnotatedTypeMirror.AnnotatedTypeVariable;

import com.sun.source.tree.CompilationUnitTree;


/**
 * The main checker class for the Generic Universe Types checker.
 *
 * @author wmdietl
 */
// Keep these synchronized with the superclass.
@TypeQualifiers({ Any.class, Peer.class, Rep.class,
    Lost.class, Self.class, Bottom.class})
@SupportedOptions( { "warn" } )
@SupportedLintOptions({"allowLost", "checkOaM", "checkStrictPurity"})
public class GUTIChecker extends GUTChecker implements InferenceTypeChecker {

    @Override
    public AnnotatedTypeFactory createFactory(CompilationUnitTree root) {
        return new GUTAnnotatedTypeFactory(this, root);
    }

    @Override
    protected GUTIVisitor createSourceVisitor(CompilationUnitTree root) {
        // The false turns off inference and enables checking the type system.
        return new GUTIVisitor(this, root, this, false);
    }

    @Override
    public boolean needsAnnotation(AnnotatedTypeMirror ty) {
        return !( ty instanceof AnnotatedPrimitiveType ||
                ty instanceof AnnotatedTypeVariable ||
                ty instanceof AnnotatedNullType);
    }

    @Override
    public AnnotationMirror defaultQualifier() {
        return this.PEER;
    }

    @Override
    public AnnotationMirror selfQualifier() {
        return this.SELF;
    }

    @Override
    public boolean withCombineConstraints() {
        return true;
    }
}